<?php
	define( 'GOOGLE_CHARTS_URL', 	'http://chart.apis.google.com/chart?' 	);

	class GoogleChart
	{
		private 	$chart_type	=	null;
		private		$chart_name	=	null;
		private		$width		=	null;
		private		$height		=	null;

		private		$data		=	Array();
		private		$high_and_low	=	Array();		// the highest and lowest value for every data set
										// used for scaling
		private		$colors		=	Array();

		function __construct( $chart_type, $chart_name = null, $width = 640, $height = 460 )
		{
			$this->chart_type	=	$chart_type;
			$this->chart_name	=	$chart_name;
			$this->width		=	$width;
			$this->height		=	$height;

			switch ( $this->chart_type )
			{
				case 'pc':
				case 'p':		// 2D pie chart needs to be twice as wide as it is high to properly
							// display labels.	
							$this->height	=	$this->width / 2;
							break;
				case 'p3':		// 3D pie chart needs to be 2.5 times as wide as it is high for labels.	
							$this->height	=	$this->width / 2.5;
							break;
			}
		}

		public function SetColor( $set_id, $color )
		{
			$this->colors[ $set_id ]	=	$color;
		}

		public function AddPoint( $set_id, $label, $value )
		{
			// add $value as the next data point in data set identified by $set_id

			$value					=	floatval( sprintf( '%0.1f', floatval( $value ) ) );
			$this->data[ $set_id ][ $label ]	=	$value;

			if ( empty( $this->high_and_low[ $set_id ] ) )
				$this->high_and_low[ $set_id ] 		=	Array( 'highest' => $value, 'lowest' => $value );
			else
			{
				$high_and_low		=	& $this->high_and_low[ $set_id ];

				if ( $value > $high_and_low[ 'highest' ] )
					$high_and_low[ 'highest' ] 	=	$value;
				if ( $value < $high_and_low[ 'lowest' ] )
					$high_and_low[ 'lowest' ]	=	$value;
			}
		}

		private function IsPieChart()
		{
			return $this->chart_type[ 0 ] == 'p';
		}

		private function GetOverallHighest()
		{
			$Highest 	=	null;
			foreach ( $this->high_and_low as $endpoints )
				if ( ( $Highest === null ) || ( $endpoints[ 'highest' ] > $Highest ) ) 	
					$Highest	=	$endpoints[ 'highest' ];

			return $Highest;
		}

		private function GetOverallLowest()
		{
			$Lowest		=	null;
			foreach ( $this->high_and_low as $endpoints )
				if ( ( $Lowest == null ) || ( $endpoints[ 'lowest' ] < $Lowest ) )
					$Lowest		=	$endpoints[ 'lowest' ];

			return $Lowest;
		}

		public function GetURL( $set_ids = null )
		{
			// pass an array of data set id's to be pulled into the URL,
			// in the order in which they need to be pulled in

			if ( $set_ids === null )
				$set_ids	=	array_keys( $this->data );

			$Parts			=		Array();
			$Parts[]		=		'chs=' . $this->width . 'x' . $this->height;
			$Parts[]		=		'cht=' . $this->chart_type;

			// data
			$DataSets		=		Array();
			// scaling parameters
			$Scaling_Params		=		Array();
			// colors
			$Colors			=		Array();
			// Labels
			$Axis			=		Array();
			$Labels			=		Array();	// for the x axis
			$Ranges			=		Array();	// for the y axis

			foreach ( $set_ids as $set_id )
			{
				if ( empty( $this->data[ $set_id ] ) ) 	continue;	// requested nonexistent data set, ignore

				$DataSets[]		=		join( ',', $this->data[ $set_id ] );

				/*
				$Scaling_Params[]	=		$this->high_and_low[ $set_id ][ 'lowest' ];
				$Scaling_Params[]	=		$this->high_and_low[ $set_id ][ 'highest' ];
				*/
				$Scaling_Params[]	=		$this->GetOverallLowest();
				$Scaling_Params[]	=		$this->GetOverallHighest();

				if ( empty( $this->colors[ $set_id ] ) )
					$Colors[]	=		-1;
				else
					$Colors[]	=		$this->colors[ $set_id ];

				if ( empty( $Labels ) )
				{
					$Axis[]		=		'x';
					$Labels[]	=		
							( $this->IsPieChart() ? '' : ( count( $Axis ) - 1 ) . ':|' ) 
								. join( '|', array_keys( $this->data[ $set_id ] ) );
						
				}

				if ( empty( $Ranges ) )
				{
					$Axis[]			=		'y';
					/*
					$Ranges[]		=		( count( $Axis ) - 1 ) . 
											',' . $this->high_and_low[ $set_id ][ 'lowest' ] .
											',' . $this->high_and_low[ $set_id ][ 'highest' ];
					*/
					$Ranges[]		=		( count( $Axis ) - 1 ) . 
											',' . $this->GetOverallLowest() . 
											',' . $this->GetOverallHighest();
				}
			}

			// actual data for chart
			$Parts[]		=		'chd=t:' . join( '|', $DataSets );	

			// Data set colors
			$Parts[]		=		'chco=' . join( ',', $Colors );

			if ( ! $this->IsPieChart() )
			{
				// Data scaling
				$Parts[]		=		'chds=' . join( ',', $Scaling_Params );	

				// Axis being used in this chart
				$Parts[]		=		'chxt=' . join( ',', $Axis );

				// Ranges being defined for particular axis
				$Parts[]		=		'chxr=' . join( '|', $Ranges );

				// legend.  t = at the top of the graph
				$Parts[]		=		'chdl=' . join( '|', $set_ids );
				$Parts[]		=		'chdlp=t';

				// axis labels
				$Parts[]		=		'chxl=' . join( '|', $Labels );
			}
			else
			{
				// pie chart segment labels
				$Parts[]		=		'chl=' . join( '|', $Labels );
			}
		
			// chart's name
			if ( !empty( $this->chart_name ) )
				$Parts[]		=		'chtt=' . urlencode( $this->chart_name );

			$url			=		GOOGLE_CHARTS_URL .
								join( '&', $Parts );

			return $url;
		}	
	}
?>
